@@ -3,7 +3,7 @@
  * Author: Brent Hendricks
  * Project: GGZ Core Client Lib
  * Date: 2/28/2001
- * $Id: game.c 7889 2006-03-07 09:57:32Z josef $
+ * $Id: game.c 8533 2006-08-26 01:26:29Z jdorje $
  *
  * This fils contains functions for handling games being played
  *
@@ -285,7 +285,7 @@ void _ggzcore_game_init(struct _GGZGame *game,
 	GGZRoom *room = _ggzcore_server_get_cur_room(server);
 
 	game->server = server;
-	game->room_id = ggzcore_room_get_id(room);
+	game->room_id = _ggzcore_room_get_id(room);
 	game->table_id = -1;
 
 	_ggzcore_server_set_cur_game(server, game);
@@ -439,7 +439,7 @@ static void _ggzcore_game_handle_state(GGZMod * mod, GGZModEvent event,
 		_ggzcore_game_send_player_stats(game);
 		_ggzcore_game_event(game, GGZ_GAME_NEGOTIATED, NULL);
 		if (new != GGZMOD_STATE_CONNECTED) {
-			ggz_error_msg("Game changed state from created"
+			ggz_error_msg("Game changed state from created "
 				      "to %d.", new);
 		}
 		break;
@@ -609,7 +609,7 @@ void _ggzcore_game_set_table(GGZGame * game, int room_id, int table_id)
 	assert(game->table_id < 0 || game->table_id == table_id);
 
 	room = ggzcore_server_get_cur_room(game->server);
-	assert(ggzcore_room_get_id(room) == room_id);
+	assert(_ggzcore_room_get_id(room) == room_id);
 
 	game->table_id = table_id;
 	table = ggzcore_room_get_table_by_id(room, table_id);
@@ -780,6 +780,8 @@ int _ggzcore_game_read_data(struct _GGZGame *game)
 		  status);
 
 	if (status < 0) {
+		/* This doesn't have to be a launch error, but may be one */
+		_ggzcore_game_event(game, GGZ_GAME_LAUNCH_FAIL, NULL);
 		abort_game(game);
 	}
 
