@@ -3,7 +3,7 @@
  * Author: Brent Hendricks
  * Project: GGZ Core Client Lib
  * Date: 6/5/00
- * $Id: room.c 7889 2006-03-07 09:57:32Z josef $
+ * $Id: room.c 8763 2006-12-27 10:02:33Z jdorje $
  *
  * This fils contains functions for handling rooms
  *
@@ -60,7 +60,8 @@ static char *_ggzcore_room_events[] = {
 	"GGZ_TABLE_LEAVE_FAIL",
 	"GGZ_PLAYER_LAG",
 	"GGZ_PLAYER_STATS",
-	"GGZ_PLAYER_COUNT"
+	"GGZ_PLAYER_COUNT",
+	"GGZ_PLAYER_PERMS"
 };
 
 /*
@@ -74,9 +75,15 @@ struct _GGZRoom {
 	/* Monitoring flag */
 	char monitor;
 
+	/* Validity flag */
+	char removal_pending;
+
 	/* Room ID on the server */
 	unsigned int id;
 
+	/* The rooms index in the server's room list. */
+	int num;
+
 	/* Name of room */
 	char *name;
 
@@ -121,36 +128,51 @@ static unsigned int _ggzcore_num_events =
  */
 
 
-static GGZServer *_ggzcore_room_get_server(GGZRoom * room)
+static GGZServer *_ggzcore_room_get_server(const GGZRoom * room)
 {
 	return room->server;
 }
 
 
-static unsigned int _ggzcore_room_get_id(const GGZRoom * room)
+unsigned int _ggzcore_room_get_id(const GGZRoom * room)
 {
 	return room->id;
 }
 
-static const char *_ggzcore_room_get_name(GGZRoom * room)
+int _ggzcore_room_get_num(const GGZRoom *room)
+{
+	return room->num;
+}
+
+void _ggzcore_room_set_num(GGZRoom *room, int num)
+{
+	room->num = num;
+}
+
+static unsigned int _ggzcore_room_get_closed(const GGZRoom * room)
+{
+	return room->removal_pending;
+}
+
+static const char *_ggzcore_room_get_name(const GGZRoom * room)
 {
 	return room->name;
 }
 
 
-static struct _GGZGameType *_ggzcore_room_get_game(GGZRoom * room)
+static struct _GGZGameType *_ggzcore_room_get_game(const GGZRoom * room)
 {
 	return _ggzcore_server_get_type_by_id(room->server, room->game);
 }
 
 
-static const char *_ggzcore_room_get_desc(GGZRoom * room)
+static const char *_ggzcore_room_get_desc(const GGZRoom * room)
 {
 	return room->desc;
 }
 
 
-static unsigned int _ggzcore_room_get_num_players(GGZRoom * room)
+static unsigned int _ggzcore_room_get_num_players(const GGZRoom * room)
 {
 	if (ggzcore_server_get_cur_room(room->server) == room) {
 		return room->num_players;
@@ -160,7 +182,7 @@ static unsigned int _ggzcore_room_get_num_players(GGZRoom * room)
 }
 
 
-static GGZPlayer *_ggzcore_room_get_nth_player(GGZRoom * room,
+static GGZPlayer *_ggzcore_room_get_nth_player(const GGZRoom * room,
 					       const unsigned int num)
 {
 	int i;
@@ -174,7 +196,7 @@ static GGZPlayer *_ggzcore_room_get_nth_player(GGZRoom * room,
 }
 
 
-GGZPlayer *_ggzcore_room_get_player_by_name(GGZRoom * room,
+GGZPlayer *_ggzcore_room_get_player_by_name(const GGZRoom * room,
 					    const char *name)
 {
 	GGZListEntry *entry;
@@ -182,7 +204,7 @@ GGZPlayer *_ggzcore_room_get_player_by_name(GGZRoom * room,
 
 	if (room->players) {
 		player = _ggzcore_player_new();
-		_ggzcore_player_init(player, name, NULL, -1, 0, 0);
+		_ggzcore_player_init(player, name, NULL, -1, 0, 0, 0);
 		entry = ggz_list_search(room->players, player);
 
 		if (entry)
@@ -194,14 +216,14 @@ GGZPlayer *_ggzcore_room_get_player_by_name(GGZRoom * room,
 }
 
 
-static unsigned int _ggzcore_room_get_num_tables(GGZRoom * room)
+static unsigned int _ggzcore_room_get_num_tables(const GGZRoom * room)
 {
 	/* FIXME: we should let the list track this instead of doing it ourselves */
 	return room->num_tables;
 }
 
 
-static struct _GGZTable *_ggzcore_room_get_nth_table(GGZRoom * room,
+static struct _GGZTable *_ggzcore_room_get_nth_table(const GGZRoom * room,
 						     const unsigned int
 						     num)
 {
@@ -216,7 +238,7 @@ static struct _GGZTable *_ggzcore_room_get_nth_table(GGZRoom * room,
 }
 
 
-static struct _GGZTable *_ggzcore_room_get_table_by_id(GGZRoom * room,
+static struct _GGZTable *_ggzcore_room_get_table_by_id(const GGZRoom * room,
 						       const unsigned int
 						       id)
 {
@@ -250,19 +272,6 @@ GGZRoom *ggzcore_room_new(void)
 }
 
 
-/* Initialize room object */
-int ggzcore_room_init(GGZRoom * room, GGZServer * server,
-		      const unsigned int id, const char *name,
-		      const unsigned int game, const char *desc)
-{
-	if (room) {
-		_ggzcore_room_init(room, server, id, name, game, desc, 0);
-		return 0;
-	} else
-		return -1;
-}
-
-
 /* De-allocate room object and its children */
 void ggzcore_room_free(GGZRoom * room)
 {
@@ -271,7 +280,7 @@ void ggzcore_room_free(GGZRoom * room)
 }
 
 
-GGZServer *ggzcore_room_get_server(GGZRoom * room)
+GGZServer *ggzcore_room_get_server(const GGZRoom * room)
 {
 	if (room)
 		return _ggzcore_room_get_server(room);
@@ -280,15 +289,15 @@ GGZServer *ggzcore_room_get_server(GGZRoom * room)
 }
 
 
-int ggzcore_room_get_id(const GGZRoom * room)
+int ggzcore_room_get_closed(const GGZRoom * room)
 {
 	if (room)
-		return _ggzcore_room_get_id(room);
+		return _ggzcore_room_get_closed(room);
 	else
 		return -1;
 }
 
-const char *ggzcore_room_get_name(GGZRoom * room)
+const char *ggzcore_room_get_name(const GGZRoom * room)
 {
 	if (room)
 		return _ggzcore_room_get_name(room);
@@ -297,7 +306,7 @@ const char *ggzcore_room_get_name(GGZRoom * room)
 }
 
 
-const char *ggzcore_room_get_desc(GGZRoom * room)
+const char *ggzcore_room_get_desc(const GGZRoom * room)
 {
 	if (room)
 		return _ggzcore_room_get_desc(room);
@@ -306,7 +315,7 @@ const char *ggzcore_room_get_desc(GGZRoom * room)
 }
 
 
-GGZGameType *ggzcore_room_get_gametype(GGZRoom * room)
+GGZGameType *ggzcore_room_get_gametype(const GGZRoom * room)
 {
 	if (room)
 		return _ggzcore_room_get_game(room);
@@ -315,7 +324,7 @@ GGZGameType *ggzcore_room_get_gametype(GGZRoom * room)
 }
 
 
-int ggzcore_room_get_num_players(GGZRoom * room)
+int ggzcore_room_get_num_players(const GGZRoom * room)
 {
 	if (room)
 		return _ggzcore_room_get_num_players(room);
@@ -324,7 +333,7 @@ int ggzcore_room_get_num_players(GGZRoom * room)
 }
 
 
-int ggzcore_room_get_num_tables(GGZRoom * room)
+int ggzcore_room_get_num_tables(const GGZRoom * room)
 {
 	if (room)
 		return _ggzcore_room_get_num_tables(room);
@@ -333,7 +342,7 @@ int ggzcore_room_get_num_tables(GGZRoom * room)
 }
 
 
-GGZPlayer *ggzcore_room_get_nth_player(GGZRoom * room,
+GGZPlayer *ggzcore_room_get_nth_player(const GGZRoom * room,
 				       const unsigned int num)
 {
 	if (room && num < room->num_players)
@@ -343,7 +352,7 @@ GGZPlayer *ggzcore_room_get_nth_player(GGZRoom * room,
 }
 
 
-GGZTable *ggzcore_room_get_nth_table(GGZRoom * room,
+GGZTable *ggzcore_room_get_nth_table(const GGZRoom * room,
 				     const unsigned int num)
 {
 	if (room && num < room->num_tables)
@@ -354,7 +363,7 @@ GGZTable *ggzcore_room_get_nth_table(GGZRoom * room,
 
 
 
-GGZTable *ggzcore_room_get_table_by_id(GGZRoom * room,
+GGZTable *ggzcore_room_get_table_by_id(const GGZRoom * room,
 				       const unsigned int id)
 {
 	if (!room)
@@ -365,7 +374,8 @@ GGZTable *ggzcore_room_get_table_by_id(GGZRoom * room,
 
 
 /* Functions for manipulating hooks to GGZRoom events */
-int ggzcore_room_add_event_hook(GGZRoom * room, const GGZRoomEvent event,
+int ggzcore_room_add_event_hook(GGZRoom * room,
+				const GGZRoomEvent event,
 				const GGZHookFunc func)
 {
 	if (room && func && _ggzcore_room_event_is_valid(event))
@@ -442,6 +452,20 @@ int ggzcore_room_chat(GGZRoom * room, const GGZChatType type,
 }
 
 
+int ggzcore_room_admin(GGZRoom *room, GGZAdminType type,
+		       const char *player, const char *reason)
+{
+	if (room && room->server) {
+		/* FIXME: checks! */
+		/* Check 1: 'player' must exist in current room */
+		/* Check 2: reason must be given for kick */
+		/* Check 3: sender must be admin (or host?) */
+		return _ggzcore_room_admin(room, type, player, reason);
+	} else
+		return -1;
+}
+
+
 int ggzcore_room_launch_table(GGZRoom * room, GGZTable * table)
 {
 	if (room && room->server && table)
@@ -485,6 +509,7 @@ GGZRoom *_ggzcore_room_new(void)
 	GGZRoom *room;
 
 	room = ggz_malloc(sizeof(GGZRoom));
+	room->removal_pending = 0;
 
 	return room;
 }
@@ -550,7 +575,7 @@ void _ggzcore_room_set_player_list(GGZRoom * room,
 	room->player_count = count;
 	room->players = list;
 
-	_ggzcore_room_event(room, GGZ_PLAYER_LIST, &room->id);
+	_ggzcore_room_event(room, GGZ_PLAYER_LIST, room);
 	if (count_changed) {
 		GGZServer *server = _ggzcore_room_get_server(room);
 
@@ -571,7 +596,7 @@ void _ggzcore_room_set_players(GGZRoom * room, int players)
 		/* Sanity check. */
 		room->player_count = 0;
 	}
-	_ggzcore_room_event(room, GGZ_PLAYER_COUNT, &room->id);
+	_ggzcore_room_event(room, GGZ_PLAYER_COUNT, room);
 	_ggzcore_server_queue_players_changed(server);
 }
 
@@ -616,7 +641,7 @@ void _ggzcore_room_set_monitor(GGZRoom * room, char monitor)
 
 
 void _ggzcore_room_add_player(GGZRoom * room, GGZPlayer * pdata,
-			      int from_room)
+			      int room_known, GGZRoom *from_room)
 {
 	GGZPlayer *player;
 	GGZRoomChangeEventData data;
@@ -644,6 +669,7 @@ void _ggzcore_room_add_player(GGZRoom * room, GGZPlayer * pdata,
 	_ggzcore_player_init(player, ggzcore_player_get_name(pdata),
 			     _ggzcore_player_get_room(pdata),
 			     -1, ggzcore_player_get_type(pdata),
+			     _ggzcore_player_get_perms(pdata),
 			     ggzcore_player_get_lag(pdata));
 	_ggzcore_player_init_stats(player, wins, losses, ties, forfeits,
 				   rating, ranking, highscore);
@@ -653,20 +679,21 @@ void _ggzcore_room_add_player(GGZRoom * room, GGZPlayer * pdata,
 	room->player_count = room->num_players;
 
 	data.player_name = ggzcore_player_get_name(pdata);
+	data.rooms_known = room_known;
 	data.from_room = from_room;
-	data.to_room = room->id;
+	data.to_room = room;
 	_ggzcore_room_event(room, GGZ_ROOM_ENTER, &data);
 
-	if ((room =
-	     _ggzcore_server_get_room_by_id(room->server, from_room))) {
-		_ggzcore_room_set_players(room, room->player_count - 1);
+	if (from_room) {
+		_ggzcore_room_set_players(from_room,
+					  from_room->player_count - 1);
 	}
 	_ggzcore_server_queue_players_changed(server);
 }
 
 
 void _ggzcore_room_remove_player(GGZRoom * room, const char *name,
-				 int to_room)
+				 int room_known, GGZRoom *to_room)
 {
 	GGZListEntry *entry;
 	GGZRoomChangeEventData data;
@@ -676,7 +703,7 @@ void _ggzcore_room_remove_player(GGZRoom * room, const char *name,
 	/* Only try to delete if the list exists */
 	if (room->players) {
 		GGZPlayer *player = _ggzcore_player_new();
-		_ggzcore_player_init(player, name, NULL, -1, 0, 0);
+		_ggzcore_player_init(player, name, NULL, -1, 0, 0, 0);
 		entry = ggz_list_search(room->players, player);
 		if (entry) {
 			GGZServer *server = _ggzcore_room_get_server(room);
@@ -686,7 +713,8 @@ void _ggzcore_room_remove_player(GGZRoom * room, const char *name,
 			room->player_count = room->num_players;
 
 			data.player_name = name;
-			data.from_room = room->id;
+			data.rooms_known = room_known;
+			data.from_room = room;
 			data.to_room = to_room;
 
 			_ggzcore_room_event(room, GGZ_ROOM_LEAVE, &data);
@@ -695,8 +723,8 @@ void _ggzcore_room_remove_player(GGZRoom * room, const char *name,
 		_ggzcore_player_free(player);
 	}
 
-	if ((room = _ggzcore_server_get_room_by_id(room->server, to_room))) {
-		_ggzcore_room_set_players(room, room->player_count + 1);
+	if (to_room) {
+		_ggzcore_room_set_players(to_room, to_room->player_count + 1);
 	}
 }
 
@@ -826,6 +854,17 @@ int _ggzcore_room_chat(GGZRoom * room,
 }
 
 
+int _ggzcore_room_admin(GGZRoom *room, GGZAdminType type,
+		       const char *player, const char *reason)
+{
+	GGZNet *net;
+
+	net = _ggzcore_server_get_net(room->server);
+
+	return _ggzcore_net_send_room_admin(net, type, player, reason);
+}
+
+
 void _ggzcore_room_add_chat(GGZRoom * room, GGZChatType type,
 			    const char *name, const char *msg)
 {
@@ -1179,6 +1218,10 @@ void _ggzcore_room_destroy(void *p)
 }
 
 
+void _ggzcore_room_close(GGZRoom *room)
+{
+	room->removal_pending = 1;
+}
 
 
 /* Static functions internal to this file */
