@@ -3,7 +3,7 @@
  * Author: Brent Hendricks
  * Project: GGZ Core Client Lib
  * Date: 1/19/01
- * $Id: server.c 7889 2006-03-07 09:57:32Z josef $
+ * $Id: server.c 8706 2006-12-06 02:44:50Z jdorje $
  *
  * Code for handling server connection state and properties
  *
@@ -40,6 +40,7 @@
 #include <winsock2.h>
 #endif
 #include <signal.h>
+#include <sys/time.h>
 #include <time.h>
 #include <unistd.h>
 
@@ -392,6 +393,15 @@ GGZRoom *ggzcore_server_get_nth_room(const GGZServer * server,
 		return NULL;
 }
 
+int ggzcore_server_get_room_num(const GGZServer *server,
+				const GGZRoom *room)
+{
+	if (server && room)
+		return _ggzcore_server_get_room_num(server, room);
+	else
+		return 0;
+}
+
 int ggzcore_server_get_num_rooms(const GGZServer * server)
 {
 	if (server)
@@ -543,14 +553,13 @@ int ggzcore_server_list_gametypes(GGZServer * server, const char verbose)
 }
 
 
-int ggzcore_server_join_room(GGZServer * server,
-			     const unsigned int room_num)
+int ggzcore_server_join_room(GGZServer * server, GGZRoom *room)
 {
 	/* FIXME: check validity of this action */
-	if (server && (room_num < server->num_rooms)
+	if (server && room
 	    && (server->state == GGZ_STATE_IN_ROOM
 		|| server->state == GGZ_STATE_LOGGED_IN))
-		return _ggzcore_server_join_room(server, room_num);
+		return _ggzcore_server_join_room(server, room);
 	else
 		return -1;
 }
@@ -701,6 +710,7 @@ int _ggzcore_server_get_tls(const GGZServer * server)
 int _ggzcore_server_get_num_players(const GGZServer * server)
 {
 	int rooms = ggzcore_server_get_num_rooms(server), i, total = 0;
+	GGZRoom *cur_room = ggzcore_server_get_cur_room(server);
 
 	for (i = 0; i < rooms; i++) {
 		GGZRoom *room = ggzcore_server_get_nth_room(server, i);
@@ -711,6 +721,12 @@ int _ggzcore_server_get_num_players(const GGZServer * server)
 		total += ggzcore_room_get_num_players(room);
 	}
 
+	if (!cur_room) {
+		/* If we aren't in a room we can assume there's at least one
+		   more player there. */
+		total++;
+	}
+
 	return total;
 }
 
@@ -743,7 +759,7 @@ GGZRoom *_ggzcore_server_get_room_by_id(const GGZServer * server,
 	int i;
 
 	for (i = 0; i < server->num_rooms; i++)
-		if (ggzcore_room_get_id(server->rooms[i]) == id)
+		if (_ggzcore_room_get_id(server->rooms[i]) == id)
 			return server->rooms[i];
 
 	return NULL;
@@ -754,6 +770,13 @@ GGZRoom *_ggzcore_server_get_nth_room(const GGZServer * server,
 				      const unsigned int num)
 {
 	return server->rooms[num];
+	/* FIXME: does this work for fragmented room lists, too? */
+}
+
+int _ggzcore_server_get_room_num(const GGZServer *server,
+				 const GGZRoom *room)
+{
+	return _ggzcore_room_get_num(room);
 }
 
 
@@ -899,7 +922,19 @@ void _ggzcore_server_set_login_status(GGZServer * server,
 			break;
 		case E_USR_LOOKUP:
 			snprintf(error.message, sizeof(error.message),
-				_("Name taken or wrong password"));
+				_("The password was incorrect"));
+			break;
+		case E_USR_TAKEN:
+			snprintf(error.message, sizeof(error.message),
+				_("Name is already taken"));
+			break;
+		case E_USR_TYPE:
+			snprintf(error.message, sizeof(error.message),
+				_("This name is already registered so cannot be used by a guest"));
+			break;
+		case E_USR_FOUND:
+			snprintf(error.message, sizeof(error.message),
+				_("No such name was found"));
 			break;
 		case E_TOO_LONG:
 			snprintf(error.message, sizeof(error.message),
@@ -1264,18 +1299,12 @@ int _ggzcore_server_load_roomlist(GGZServer * server, const int type,
 }
 
 
-int _ggzcore_server_join_room(GGZServer * server,
-			      const unsigned int room_num)
+int _ggzcore_server_join_room(GGZServer * server, GGZRoom *room)
 {
 	int status;
-	int room_id;
-	GGZRoom *room;
+	int room_id = _ggzcore_room_get_id(room);
 
-	/* We need to send the room's ID on the server */
-	room = _ggzcore_server_get_nth_room(server, room_num);
-	room_id = ggzcore_room_get_id(room);
-
-	ggz_debug(GGZCORE_DBG_SERVER, "Moving to room %d", room_num);
+	ggz_debug(GGZCORE_DBG_SERVER, "Moving to room %d", room_id);
 
 	status = _ggzcore_net_send_join_room(server->net, room_id);
 	server->new_room = room;
@@ -1459,6 +1488,7 @@ void _ggzcore_server_add_room(GGZServer * server, GGZRoom * room)
 	while (i < server->num_rooms) {
 		if (server->rooms[i] == NULL) {
 			server->rooms[i] = room;
+			_ggzcore_room_set_num(room, i);
 			break;
 		}
 		++i;
@@ -1466,6 +1496,43 @@ void _ggzcore_server_add_room(GGZServer * server, GGZRoom * room)
 }
 
 
+void _ggzcore_server_delete_room(GGZServer * server, GGZRoom * room)
+{
+	int i = 0;
+	int j;
+
+	/* Find room with the same id, and empty its slot */
+	while (i < server->num_rooms) {
+		if (server->rooms[i] != NULL) {
+			if (!_ggzcore_room_compare(server->rooms[i], room)) {
+				_ggzcore_room_free(server->rooms[i]);
+				/*server->rooms[i] = NULL;*/
+				server->num_rooms--;
+				/* FIXME: what if not at end? reorder! */
+				for (j = i; j < server->num_rooms; j++) {
+					server->rooms[j] = server->rooms[j + 1];
+					_ggzcore_room_set_num(server->rooms[j], j);
+				}
+				server->rooms[server->num_rooms] = NULL;
+				break;
+			}
+		}
+		++i;
+	}
+}
+
+
+void _ggzcore_server_grow_roomlist(GGZServer * server)
+{
+	int num;
+
+	num = server->num_rooms;
+	server->num_rooms++;
+	server->rooms = ggz_realloc(server->rooms, (num + 1) * sizeof(*server->rooms));
+	server->rooms[num] = NULL;
+}
+
+
 void _ggzcore_server_init_typelist(GGZServer * server, const int num)
 {
 	server->num_gametypes = num;
