@@ -4,7 +4,7 @@
  * Project: ggzmod
  * Date: 10/14/01
  * Desc: GGZ game module functions
- * $Id: ggzmod.c 7888 2006-03-07 09:57:17Z josef $
+ * $Id: ggzmod.c 8837 2007-01-05 12:49:29Z josef $
  *
  * This file contains the backend for the ggzmod library.  This
  * library facilitates the communication between the GGZ core client (ggz)
@@ -60,11 +60,12 @@
 #include "ggzcore-ggz.h"
 
 #include "ggzmod.h"
-/*#include "ggzmod-ggz.h"*/
 #include "mod.h"
 #include "io.h"
 #include "protocol.h"
 
+/* See ggzmod-ggz.h for an explanation. */
+#define GGZMOD_DEFAULT_FD 53
 
 /* 
  * internal function prototypes
@@ -140,6 +141,20 @@ static void infos_free(void *pstat)
 	ggz_free(info);
 }
 
+
+int ggzmod_is_ggz_mode(void)
+{
+	char *ggzmode;
+
+#ifdef HAVE_GETENV
+	ggzmode = getenv("GGZMODE");
+#else
+	ggzmode = GetEnvironmentVariable("GGZMODE");
+#endif
+	return (ggzmode && strcmp(ggzmode, "true") == 0);
+}
+
+
 /*
  * How a game is launched (incomplete):
  *
@@ -638,7 +653,6 @@ void _ggzmod_handle_info(GGZMod * ggzmod, int seat_num, const char *realname,
 			call_handler(ggzmod, GGZMOD_EVENT_INFO, &info);
 		}
 	}
-
 }
 
 
@@ -665,10 +679,10 @@ int ggzmod_connect(GGZMod * ggzmod)
 		if(ggzsocketstr) {
 			items = sscanf(ggzsocketstr, "%d", &ggzsocket);
 			if (items == 0) {
-				ggzsocket = 103;
+				ggzsocket = GGZMOD_DEFAULT_FD;
 			}
 		} else {
-			ggzsocket = 103;
+			ggzsocket = GGZMOD_DEFAULT_FD;
 		}
 
 #ifdef HAVE_SOCKETPAIR
